@@ -1,6 +1,5 @@
 """Markdown files translator using PO files as reference."""
 
-import glob
 import re
 import textwrap
 
@@ -11,17 +10,23 @@ from mdpo.command import (
     normalize_mdpo_command_aliases,
     parse_mdpo_html_command,
 )
-from mdpo.io import filter_paths, to_file_content_if_is_file
+from mdpo.io import to_file_content_if_is_file
 from mdpo.md import (
     escape_links_titles,
     fixwrap_codespans,
     inline_untexted_links,
+    parse_link_references,
 )
 from mdpo.md4c import DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS
-from mdpo.po import po_escaped_string
+from mdpo.po import (
+    paths_or_globs_to_unique_pofiles,
+    po_escaped_string,
+    pofiles_to_unique_translations_dicts,
+)
 from mdpo.polib import *  # noqa
 from mdpo.text import (
     min_not_max_chars_in_a_row,
+    removesuffix,
     wrap_different_first_line_width,
 )
 
@@ -30,6 +35,7 @@ class Po2Md:
     __slots__ = (
         'pofiles',
         'output',
+        'content',
         'extensions',
         'disabled_entries',
         'translated_entries',
@@ -67,7 +73,6 @@ class Po2Md:
         '_enterspan_replacer',
         '_leavespan_replacer',
 
-
         # state
         '_inside_htmlblock',
         '_inside_codeblock',
@@ -80,6 +85,7 @@ class Po2Md:
         '_codespan_backticks',
         '_codespan_inside_current_msgid',
         '_inside_quoteblock',
+        '_current_aspan_target',
         '_current_aspan_href',
         '_current_imgspan',
         '_current_thead_aligns',
@@ -88,13 +94,15 @@ class Po2Md:
         '_ol_marks',
         '_current_list_type',
         '_current_wikilink_target',
+        '_link_references',
     )
 
     def __init__(self, pofiles, ignore=[], po_encoding=None, **kwargs):
-        self.pofiles = [
-            polib.pofile(po_filepath, encoding=po_encoding) for po_filepath in
-            filter_paths(glob.glob(pofiles), ignore_paths=ignore)
-        ]
+        self.pofiles = paths_or_globs_to_unique_pofiles(
+            pofiles,
+            ignore,
+            po_encoding=po_encoding,
+        )
 
         self.extensions = kwargs.get(
             'extensions',
@@ -194,7 +202,9 @@ class Po2Md:
 
         self._inside_quoteblock = False
 
+        self._current_aspan_target = None
         self._current_aspan_href = None
+        self._link_references = None
         self._current_imgspan = {}
 
         # current table head alignments
@@ -264,18 +274,15 @@ class Po2Md:
         except KeyError:
             return msgid
         else:
-            if msgstr:
-                self.translated_entries.append(
-                    polib.POEntry(
-                        msgid=msgid,
-                        msgctxt=msgctxt,
-                        msgstr=msgstr,
-                        tcomment=tcomment,
-                    ),
-                )
-                return msgstr
-            else:
-                return msgid
+            self.translated_entries.append(
+                polib.POEntry(
+                    msgid=msgid,
+                    msgctxt=msgctxt,
+                    msgstr=msgstr,
+                    tcomment=tcomment,
+                ),
+            )
+            return msgstr or msgid
 
     def _save_current_msgid(self):
         if (not self._disable and not self._disable_next_line) or \
@@ -429,16 +436,31 @@ class Po2Md:
             else:
                 # inside lists, the separator '---' can't be used
                 self._current_line += '***'
+        elif block.value == md4c.BlockType.TR:
+            self._current_line += '   ' * len(self._current_list_type)
+            if self._current_line.startswith('>    '):
+                self._current_line = self._current_line.replace('> ', '')
         elif block.value == md4c.BlockType.TH:
+            if self._inside_quoteblock:
+                if not self._current_line.replace(' ', '') == '>':
+                    self._current_line = removesuffix(self._current_line, '> ')
             self._current_line += '| '
             self._current_thead_aligns.append(details['align'].value)
         elif block.value == md4c.BlockType.TD:
+            if self._inside_quoteblock:
+                if not self._current_line.replace(' ', '') == '>':
+                    self._current_line = removesuffix(self._current_line, '> ')
             self._current_line += '| '
         elif block.value == md4c.BlockType.QUOTE:
             if self._inside_liblock:
                 self._save_current_msgid()
                 self._save_current_line()
             self._inside_quoteblock = True
+        elif block.value == md4c.BlockType.TABLE:
+            if self._current_list_type and not self._inside_quoteblock:
+                if self._current_line:
+                    self._save_current_line()
+                self._save_current_line()
         elif block.value == md4c.BlockType.HTML:
             self._inside_htmlblock = True
 
@@ -533,11 +555,15 @@ class Po2Md:
             thead_separator = ''
             if self._inside_quoteblock:
                 _thead_split = re.split(r'[^\\](\|)', self._current_line)
+                if self._current_list_type:
+                    _thead_split = _thead_split[1:]
                 self._current_line += '|'
                 thead_separator += '> '
             else:
                 self._current_line += '|'
                 _thead_split = re.split(r'[^\\](\|)', self._current_line)
+                if self._current_list_type:
+                    _thead_split = _thead_split[1:-1]
             thead_separator += '| '
 
             _antepenultimate_thead_i = len(_thead_split) - 2
@@ -554,7 +580,10 @@ class Po2Md:
                 if i < len(_thead_split) - 3:
                     thead_separator += ' '
 
-            self._current_line += '\n%s' % thead_separator
+            self._current_line += '\n{}{}'.format(
+                '   ' * len(self._current_list_type),
+                thead_separator,
+            )
             self._save_current_line()
         elif block.value == md4c.BlockType.QUOTE:
             if self._outputlines[-1] == '>':
@@ -563,10 +592,8 @@ class Po2Md:
                 self._save_current_line()
             self._inside_quoteblock = False
         elif block.value == md4c.BlockType.TABLE:
-            if not self._inside_quoteblock:
+            if not self._inside_quoteblock and not self._current_list_type:
                 self._save_current_line()
-        elif block.value == md4c.BlockType.DOC:
-            pass
         elif block.value == md4c.BlockType.HTML:
             self._inside_htmlblock = False
 
@@ -579,7 +606,27 @@ class Po2Md:
             pass
 
         if span.value == md4c.SpanType.A:
+            if self._link_references is None:
+                self._link_references = parse_link_references(self.content)
+
             self._current_aspan_href = details['href'][0][1]
+            self._current_aspan_target = None
+
+            if details['title']:
+                current_aspan_title = details['title'][0][1]
+                for target, href, title in self._link_references:
+                    if (
+                        href == self._current_aspan_href and
+                        title == current_aspan_title
+                    ):
+                        self._current_aspan_target = target
+                        break
+            else:
+                for target, href, title in self._link_references:
+                    if href == self._current_aspan_href:
+                        self._current_aspan_target = target
+                        break
+
         elif span.value == md4c.SpanType.CODE:
             self._inside_codespan = True
             self._codespan_start_index = len(self._current_msgid)-1
@@ -605,17 +652,19 @@ class Po2Md:
             pass
 
         if span.value == md4c.SpanType.A:
-            if self._current_aspan_href:
-                # is not self-referenced link
-                self._current_msgid += '(%s' % self._current_aspan_href
-
-                if details['title']:
-                    self._aimg_title_inside_current_msgid = True
-                    self._current_msgid += ' "%s"' % polib.escape(
-                        details['title'][0][1],
-                    )
-                self._current_msgid += ')'
-                self._current_aspan_href = None
+            if self._current_aspan_target:
+                self._current_msgid += f'[{self._current_aspan_target}]'
+                self._current_aspan_target = None
+            else:
+                if self._current_aspan_href:
+                    self._current_msgid += '({}'.format(details['href'][0][1])
+                    if details['title']:
+                        self._aimg_title_inside_current_msgid = True
+                        self._current_msgid += ' "{}"'.format(
+                            polib.escape(details['title'][0][1]),
+                        )
+                    self._current_msgid += ')'
+            self._current_aspan_href = None
         elif span.value == md4c.SpanType.CODE:
             self._inside_codespan = False
             self._current_msgid += (
@@ -665,10 +714,10 @@ class Po2Md:
 
                 if self._inside_pblock:
                     text = text.replace('\n', ' ')
-                if self._current_aspan_href:
-                    if text == self._current_aspan_href:
-                        # self-referenced link
-                        self._current_aspan_href = None
+                if text == self._current_aspan_href:
+                    # self-referenced wiki or inline link
+                    self._current_aspan_href = None
+
                 elif self._current_wikilink_target:
                     if text != self._current_wikilink_target:
                         self._current_wikilink_target = '{}|{}'.format(
@@ -677,7 +726,6 @@ class Po2Md:
                     return
                 self._current_msgid += text
             else:
-
                 if self._inside_liblock:
                     indent = '   ' * len(self._current_list_type)
                     if self._current_line[:len(indent)+1] != indent:
@@ -686,40 +734,60 @@ class Po2Md:
         else:
             self._process_command(text)
 
-    def translate(self, filepath_or_content, save=None, md_encoding='utf-8'):
-        content = to_file_content_if_is_file(
+    def _append_link_references(self):
+        if self._link_references:
+            self._disable_next_line = False
+            self._disable = False
+
+            _references_added = []  # don't repeat references
+            for target, href, title in self._link_references:
+                href_part = '{}{}'.format(
+                    f' {href}' if href else '',
+                    f' "{title}"' if title else '',
+                )
+                if href_part in _references_added:
+                    continue
+
+                msgid = '{}{}'.format(f'[{target}]:', href_part)
+                self._outputlines.append(
+                    self._translate_msgid(msgid, None, None),
+                )
+                _references_added.append(href_part)
+            self._outputlines.append('')
+
+    def translate(
+        self,
+        filepath_or_content,
+        save=None,
+        md_encoding='utf-8',
+    ):
+        self.content = to_file_content_if_is_file(
             filepath_or_content,
             encoding=md_encoding,
         )
 
-        self.translations = {}
-        self.translations_with_msgctxt = {}
-        for pofile in self.pofiles:
-            for entry in pofile:
-                if entry.msgctxt:
-                    if entry.msgctxt not in self.translations_with_msgctxt:
-                        self.translations_with_msgctxt[entry.msgctxt] = {}
-                    self.translations_with_msgctxt[
-                        entry.msgctxt
-                    ][entry.msgid] = entry.msgstr
-                else:
-                    self.translations[entry.msgid] = entry.msgstr
+        self.translations, self.translations_with_msgctxt = (
+            pofiles_to_unique_translations_dicts(self.pofiles)
+        )
 
         parser = md4c.GenericParser(
             0,
             **{ext: True for ext in self.extensions},
         )
         parser.parse(
-            content,
+            self.content,
             self.enter_block,
             self.leave_block,
             self.enter_span,
             self.leave_span,
             self.text,
         )
+        self._append_link_references()
+
         self._disable_next_line = False
         self._disable = False
         self._enable_next_line = False
+        self._link_references = None
 
         self.output = '\n'.join(self._outputlines)
 
@@ -749,8 +817,9 @@ def pofile_to_markdown(
 
     Args:
         filepath_or_content (str): Markdown filepath or content to translate.
-        pofiles (str): Glob matching a set of pofiles from where to extract
-            references to make the replacements translating strings.
+        pofiles (str, list) Glob or list of globs matching a set of pofiles
+            from where to extract messages to make the replacements translating
+            strings.
         ignore (list): Paths of pofiles to ignore. Useful when a glob does not
             fit your requirements indicating the files to extract content.
             Also, filename or a dirname can be defined without indicate the
